
sbit Chip_Select_Direction at TRISE0_bit;
sbit Chip_Select at LATE0_bit;
sbit Chip_Select1_Direction   at TRISA2_bit;
sbit Chip_Select1             at LATA2_bit;
sbit output_sw at LATC0_bit;
sbit output_sw_Direction at TRISC0_bit;



// Lcd module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;
sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End Lcd module connections



// Variable declaration
unsigned int value;
char str[16];
unsigned int input,regout,output;
int vin;

// Get ADC values, returns 0..4095
unsigned int getADC(unsigned short channel)
{

  unsigned int tmp;
  
  Chip_Select1 = 0;                               // Select MCP3204
  SPI1_Write(0x06);                              // SPI communication using 8-bit segments
  channel = channel << 6;                        // Bits 7 & 6 define ADC input
  tmp = SPI1_Read(channel) & 0x0F;               // Get first 8 bits of ADC value
  tmp = tmp << 8;                                // Shift ADC value by 8
  tmp = tmp | SPI1_Read(0);                      // Get remaining 4 bits of ADC value
                                                 //   and form 12-bit ADC value
  Chip_Select1= 1;                                // Deselect MCP3204
  return tmp;                                    // Returns 12-bit ADC value

}



// Write measured values to Lcd

void processValue(unsigned int pv, unsigned short channel)
{
    char i1;
    char lcdRow, lcdCol;
    if (channel == 0){ lcdCol = 1;  lcdRow = 2;}
    if (channel == 1){ lcdCol = 6;  lcdRow = 1;}
    if (channel == 2){ lcdCol = 11;  lcdRow = 2;}
    // Converting the measured value into 4 characters
    // and writing them to the Lcd at the appropriate place
    i1 = pv / 1000 + 48;              // Get thousandth digit of the ADC result
    Lcd_Chr(lcdRow, lcdCol, i1);      // Display it on Lcd
    pv = pv % 1000;
    i1 = pv / 100 + 48;               // Get hundreth digit of the ADC result
    Lcd_Chr(lcdRow, lcdCol+1, i1);    // Display it on Lcd
    Lcd_Chr(lcdRow, lcdCol+2, '.');
    pv = pv % 100;
    i1 = pv / 10 + 48;                // Get tenth digit of the ADC result
    Lcd_Chr(lcdRow, lcdCol+3, i1);    // Display it on Lcd
    pv = pv % 10;
    i1 = pv + 48;                     // Get ones digit of the ADC result
    Lcd_Chr(lcdRow, lcdCol+4, i1);    // Display it on Lcd
    Lcd_Chr(lcdRow, lcdCol+5, 'V');
}



// DAC increments (0..4095) --> output voltage (0..Vref)

void DAC_Output(unsigned int valueDAC)
{
    char temp;
    Chip_Select = 0;                   // Select DAC chip
    // Send High Byte
    temp = (valueDAC >> 8) & 0x0F;     // Store valueDAC[11..8] to temp[3..0]
    temp |= 0x30;                      // Define DAC setting, see MCP4921 datasheet
    SPI1_Write(temp);                  // Send high byte via SPI
    // Send Low Byte
    temp = valueDAC;                   // Store valueDAC[7..0] to temp[7..0]
    SPI1_Write(temp);                  // Send low byte via SPI
    Chip_Select = 1;                   // Deselect DAC chip
}


void main(){

  int set_value, vin;
  bit reset;
  
  ANSELA = 0;
  ANSELE = 0;                     // Configure PORTA pins as digital
  ANSELB = 0;                        // Configure PORTB pins as digital
  anselc = 0;
  anseld = 0;
  trisd = 0xff;
  SLRCON = 0;                        // Configure all PORTS at the standard Slew rate
  value = 0;

  // Configure DAC chip select line

  Chip_Select_Direction = 0;         // Set CS# pin as Output
  Chip_Select = 1;                   // Deselect DAC

  Chip_Select1 = 1;                 // Deselect MCP3204
  Chip_Select1_Direction = 0;
  output_sw_Direction = 0;
  reset = 0;
  vin = 0;
  input = 0;
  regout = 0;
  output = 0;
  // Configure user buttons
  TRISA0_bit = 1;                    // Set RA0 pin as input
  TRISA1_bit = 1;                    // Set RA1 pin as input
  set_value = 0;
  // Initialize SPI1 module
  SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV64, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);

  Lcd_Init();                        // Initialize Lcd
  ADC_Init();
  Lcd_Cmd(_LCD_CLEAR);               // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off


  Lcd_Cmd(_LCD_CLEAR);               // Clear display
  Lcd_Out(1,1,"IN           OUT");
  Lcd_Out(2,1,"       RG       ");

  // Set inital DAC output value to the mid range

  value = 600;
  set_value = 950;

  while(1)
  {                                // Endless loop

      if(RD5_bit == 1){ output_sw = 1;}
      if(RD5_bit == 0){ output_sw = 0; reset = 0;}
      
      // If RA0 button is pressed
      if ((RA0_bit) && (value < 4095))
      {
          set_value++;                              //   increment value
      }

       // If RA1 button is pressed
      else if ((RA1_bit) && (value > 0))
      {
              set_value--;                            //   decrement value
      }

      if(RD0_bit){set_value = 180;}
      if(RD1_bit){set_value = 330;}
      if(RD2_bit){set_value = 900;}
      if(RD3_bit){set_value = 1400;}

      regout = getADC(0)/1.82;       // Get ADC result from Channel 0
      ProcessValue(regout,1);        // Writes measured value to Lcd
      Delay_ms(10);                  // Wait 10ms
      input = getADC(2)/1.82;        // Get ADC result from Channel 2
      vin = input - 150;
      ProcessValue(input,0);         // Writes measured value to Lcd
      Delay_ms(10);                  // Wait 10ms


      if(reset == 0)
          value = 0;
      
    if(output_sw == 1)
    {
        reset = 1;
      
        while((output - set_value) > 50 | (set_value - output) > 50)
        {

            output = getADC(1)/1.82;       // Get ADC result from Channel 1
            if(vin < set_value)
            {
                Lcd_Cmd(_LCD_CLEAR);
                LCD_OUT(1,1,"NOT ENOUGH");
                LCD_OUT(2,1,"INPUT VOLTS");
                output_sw = 0;
                while(vin < set_value)
                {
                    input = getADC(2)/1.82;       // Get ADC result from Channel 2
                    vin = input - 200;
                }
                
                Lcd_Cmd(_LCD_CLEAR);               // Clear display
                Lcd_Out(1,1,"IN           OUT");
                Lcd_Out(2,1,"       RG       ");
                output_sw = 1;
            }
            
            if(output < set_value)
            {
                value = value + 1;
                if(value>4095){value = 4095;}
            }
        
            if(output > set_value)
            {
                value = value - 1;
                if(value<1){value = 2;}
            }
            
            DAC_Output(value);
      }

    }
    
    if(output < set_value)
    {
        value = value + 1;
        if(value>4095) {value = 4095;}
    }
    
    if(output > set_value)
    {
        value = value - 1;
        if(value<1) {value = 2;}
    }
    
    DAC_Output(value);
    output = getADC(1)/1.82;
    ProcessValue(output,2);   // Writes measured value to Lcd
    Delay_ms(10);                  // Wait 10ms
  }

}